-- Indicator profile initialization routine
-- Defines indicator profile properties and indicator parameters
-- TODO: Add minimal and maximal value of numeric parameters and default color of the streams
function Init()
    indicator:name(resources:get("name"));
    indicator:description(resources:get("description"));
    indicator:requiredSource(core.Bar);
    indicator:type(core.Indicator);

    indicator.parameters:addInteger("TP", resources:get("param_TP_name"), resources:get("param_TP_description"), 9);
    indicator.parameters:addInteger("KP", resources:get("param_KP_name"), resources:get("param_KP_description"), 26);
    indicator.parameters:addInteger("SP", resources:get("param_SP_name"), resources:get("param_SP_description"), 52);
    indicator.parameters:addColor("TS_color", resources:get("param_TS_color_name"), resources:get("param_TS_color_description"), core.rgb(255, 0, 0));
    indicator.parameters:addColor("KS_color", resources:get("param_KS_color_name"), resources:get("param_KS_color_description"), core.rgb(0, 0, 255));
    indicator.parameters:addColor("CS_color", resources:get("param_CS_color_name"), resources:get("param_CS_color_description"), core.rgb(0, 255, 0));
    indicator.parameters:addColor("SA_color", resources:get("param_SA_color_name"), resources:get("param_SA_color_description"), core.rgb(255, 128, 128));
    indicator.parameters:addColor("SB_color", resources:get("param_SB_color_name"), resources:get("param_SB_color_description"), core.rgb(255, 128, 0));
    indicator.parameters:addColor("SgU_color", resources:get("param_SgU_color_name"), resources:get("param_SgU_color_description"), core.rgb(0, 255, 0));
    indicator.parameters:addColor("SgD_color", resources:get("param_SgD_color_name"), resources:get("param_SgD_color_description"), core.rgb(255, 0, 0));
    indicator.parameters:addColor("SgC_color", resources:get("param_SgC_color_name"), resources:get("param_SgC_color_description"), core.rgb(255, 255, 0));
end

-- Indicator instance initialization routine
-- Processes indicator parameters and creates output streams
-- TODO: Refine the first period calculation for each of the output streams.
-- TODO: Calculate all constants, create instances all subsequent indicators and load all required libraries
-- Parameters block
local TP;
local KP;
local SP;

local first;
local source = nil;

-- Streams block
local TS = nil;
local KS = nil;
local CS = nil;
local SA = nil;
local SB = nil;
local SgU = nil;
local SgD = nil;

-- Routine
function Prepare()
    TP = instance.parameters.TP;
    KP = instance.parameters.KP;
    SP = instance.parameters.SP;
    source = instance.source;
    first = source:first();

    local name = profile:id() .. "(" .. source:name() .. ", " .. TP .. ", " .. KP .. ", " .. SP .. ")";
    instance:name(name);
    TS = instance:addStream("TS", core.Line, name .. ".TS", "TS", instance.parameters.TS_color, first + TP - 1);
    KS = instance:addStream("KS", core.Line, name .. ".KS", "KS", instance.parameters.KS_color, first + KP - 1);
    CS = instance:addStream("CS", core.Line, name .. ".CS", "CS", instance.parameters.CS_color, first, -KP);
    SA = instance:addStream("SA", core.Line, name .. ".SA", "SA", instance.parameters.SA_color, math.max(TS:first(), KS:first()), KP);
    SB = instance:addStream("SB", core.Line, name .. ".SB", "SB", instance.parameters.SB_color, first + SP - 1, KP);
    SgU = instance:addStream("SgU", core.Dot, name .. ".SgU", "SgU", instance.parameters.SgU_color, first + KP - 1, KP);
    SgD = instance:addStream("SgD", core.Dot, name .. ".SgD", "SgD", instance.parameters.SgD_color, first + KP - 1, KP);
    SgC = instance:addStream("SgC", core.Dot, name .. ".SgC", "SgC", instance.parameters.SgC_color, first + KP - 1, KP);
end

-- Indicator calculation routine
-- TODO: Add your code for calculation output values
function Update(period)
    if (period >= CS:first() + KP) then
        CS[period - KP] = source.close[period];
    end

    local p, hh, ll, i;

    if (period >= TS:first()) then
        p = core.rangeTo(period, TP);
        hh, i = core.max(source.high, p);
        ll, i = core.min(source.low, p);
        TS[period] = (hh + ll) / 2;
    end

    if (period >= KS:first()) then
        p = core.rangeTo(period, KP);
        hh, i = core.max(source.high, p);
        ll, i = core.min(source.low, p);
        KS[period] = (hh + ll) / 2;
    end

    if (period >= SA:first()) then
        SA[period + KP] = (TS[period] + KS[period]) / 2;
    end

    if (period >= SB:first()) then
        p = core.rangeTo(period, SP);
        hh, i = core.max(source.high, p);
        ll, i = core.min(source.low, p);
        SB[period + KP] = (hh + ll) / 2;
    end

    if (period >= SB:first() + KP) then
        local pos;
        if (source.close[period]>=TS[period] and TS[period]>=KS[period] and (KS[period]>=SA[period] and KS[period]>=SB[period]) and CS[period - KP]>source.high[period-KP]) then
            pos = math.min(KS[period],SA[period],SB[period],source.low[period]);
            SgU[period] = pos - (pos * 0.0005);
        end
        if (source.close[period]<=TS[period] and TS[period]<=KS[period] and (KS[period]<=SA[period] and KS[period]<=SB[period]) and CS[period - KP]<source.low[period-KP]) then
            pos = math.max(KS[period],SA[period],SB[period],source.high[period]);
            SgD[period] = pos + (pos * 0.0005);
        end
        if (core.crossesOver(TS,KS,period) and (KS[period]>=SA[period] and KS[period]>=SB[period]) and CS[period - KP]>source.high[period-KP]) then
            pos = math.min(KS[period],SA[period],SB[period],source.low[period]);
            SgC[period] = pos - (pos * 0.0005);
        end
        if (core.crossesUnder(TS,KS,period) and (KS[period]<=SA[period] and KS[period]<=SB[period]) and CS[period - KP]<source.low[period-KP]) then
            pos = math.max(KS[period],SA[period],SB[period],source.high[period]);
            SgC[period] = pos + (pos * 0.0005);
        end
    end
end











